@@ -20,14 +20,11 @@ import collections
 import typing
 from typing import Any, Callable, Mapping, Optional, Sequence, Text, Union
 
-from absl import flags
 from absl import logging
 from dm_control import composer
 from dm_control.composer.observation import observable as base_observable
 from dm_control.locomotion.mocap import loader
 
-
-
 from dm_control.locomotion.tasks.reference_pose import datasets
 from dm_control.locomotion.tasks.reference_pose import types
 from dm_control.locomotion.tasks.reference_pose import utils
@@ -39,22 +36,16 @@ from dm_control.utils import transformations as tr
 from dm_env import specs
 
 import numpy as np
-import six
 import tree
 
 if typing.TYPE_CHECKING:
-  from dm_control.locomotion.walkers import base
+  from dm_control.locomotion.walkers import legacy_base
   from dm_control import mjcf
 
 mjlib = mjbindings.mjlib
-FLAGS = flags.FLAGS
 DEFAULT_PHYSICS_TIMESTEP = 0.005
 _MAX_END_STEP = 10000
 
-# Spanner query load spreading parameters.
-_TASKS_PER_BIN = 20
-_SECONDS_PER_BIN = 5
-
 
 def _strip_reference_prefix(dictionary: Mapping[Text, Any], prefix: Text):
   new_dictionary = dict()
@@ -66,25 +57,26 @@ def _strip_reference_prefix(dictionary: Mapping[Text, Any], prefix: Text):
   return new_dictionary
 
 
-@six.add_metaclass(abc.ABCMeta)
-class ReferencePosesTask(composer.Task):
+class ReferencePosesTask(composer.Task, metaclass=abc.ABCMeta):
   """Abstract base class for task that uses reference data."""
 
   def __init__(
       self,
-      walker: Union['base.Walker', Callable],  # pylint: disable=g-bare-generic
+      walker: Callable[..., 'legacy_base.Walker'],
       arena: composer.Arena,
       ref_path: Text,
       ref_steps: Sequence[int],
       dataset: Union[Text, types.ClipCollection],
       termination_error_threshold: float = 0.3,
       min_steps: int = 10,
-      reward_type: Text = 'default',
+      reward_type: Text = 'termination_reward',
       physics_timestep: float = DEFAULT_PHYSICS_TIMESTEP,
       always_init_at_clip_start: bool = False,
       proto_modifier: Optional[Any] = None,
       ghost_offset: Optional[Sequence[Union[int, float]]] = None,
       body_error_multiplier: Union[int, float] = 1.0,
+      actuator_force_coeff: float = 0.015,
+      enabled_reference_observables: Optional[Sequence[Text]] = None,
   ):
     """Abstract task that uses reference data.
 
@@ -112,6 +104,9 @@ class ReferencePosesTask(composer.Task):
         the reference pose at the specified position offset.
       body_error_multiplier: A multiplier that is applied to the body error term
         when determining failure termination condition.
+      actuator_force_coeff: A coefficient for the actuator force reward channel.
+      enabled_reference_observables: Optional iterable of enabled observables.
+        If not specified, a reasonable default set will be enabled.
     """
     self._ref_steps = np.sort(ref_steps)
     self._max_ref_step = self._ref_steps[-1]
@@ -122,6 +117,7 @@ class ReferencePosesTask(composer.Task):
     self._always_init_at_clip_start = always_init_at_clip_start
     self._ghost_offset = ghost_offset
     self._body_error_multiplier = body_error_multiplier
+    self._actuator_force_coeff = actuator_force_coeff
     logging.info('Reward type %s', reward_type)
 
     if isinstance(dataset, Text):
@@ -158,14 +154,16 @@ class ReferencePosesTask(composer.Task):
     walker_bodies_names = [bdy.name for bdy in walker_bodies]
     self._body_idxs = np.array(
         [walker_bodies_names.index(bdy) for bdy in walker_bodies_names])
+
     # Create the observables.
-    self._add_observables()
+    self._add_observables(enabled_reference_observables)
 
     # initialize counters etc.
     self._time_step = 0
     self._current_start_time = 0.0
     self._last_step = 0
     self._current_clip_index = 0
+    self._reference_observations = dict()
     self._end_mocap = False
     self._should_truncate = False
 
@@ -181,7 +179,7 @@ class ReferencePosesTask(composer.Task):
         lambda x: x[0], self._clip_reference_features)
 
     self._current_reference_features = dict()
-
+    self._reference_ego_bodies_quats = collections.defaultdict(dict)
     # if requested add ghost body to visualize motion capture reference.
     if self._ghost_offset is not None:
       self._ghost = utils.add_walker(walker, self._arena, 'ghost', ghost=True)
@@ -211,40 +209,47 @@ class ReferencePosesTask(composer.Task):
     else:
       self._all_clips = [None] * self._num_clips
 
-  def _add_observables(self):
-    observables = []
-    observables += self._walker.observables.proprioception
-    observables += self._walker.observables.kinematic_sensors
-    observables += self._walker.observables.dynamic_sensors
+  def _add_observables(self, enabled_reference_observables):
 
-    for observable in observables:
-      observable.enabled = True
-    self._walker.observables.add_observable(
-        'clip_id', base_observable.Generic(self.get_clip_id))
+    # pylint: disable=g-long-lambda
     self._walker.observables.add_observable(
         'reference_rel_joints',
-        base_observable.Generic(self.get_reference_rel_joints))
+        base_observable.Generic(lambda _: self._reference_observations[
+            'walker/reference_rel_joints']))
     self._walker.observables.add_observable(
         'reference_rel_bodies_pos_global',
-        base_observable.Generic(self.get_reference_rel_bodies_pos_global))
+        base_observable.Generic(lambda _: self._reference_observations[
+            'walker/reference_rel_bodies_pos_global']))
     self._walker.observables.add_observable(
         'reference_rel_bodies_quats',
-        base_observable.Generic(self.get_reference_rel_bodies_quats))
+        base_observable.Generic(lambda _: self._reference_observations[
+            'walker/reference_rel_bodies_quats']))
     self._walker.observables.add_observable(
         'reference_rel_bodies_pos_local',
-        base_observable.Generic(self.get_reference_rel_bodies_pos_local))
+        base_observable.Generic(lambda _: self._reference_observations[
+            'walker/reference_rel_bodies_pos_local']))
     self._walker.observables.add_observable(
         'reference_ego_bodies_quats',
-        base_observable.Generic(self.get_reference_ego_bodies_quats))
+        base_observable.Generic(lambda _: self._reference_observations[
+            'walker/reference_ego_bodies_quats']))
     self._walker.observables.add_observable(
         'reference_rel_root_quat',
-        base_observable.Generic(self.get_reference_rel_root_quat))
+        base_observable.Generic(lambda _: self._reference_observations[
+            'walker/reference_rel_root_quat']))
     self._walker.observables.add_observable(
         'reference_rel_root_pos_local',
-        base_observable.Generic(self.get_reference_rel_root_pos_local))
+        base_observable.Generic(lambda _: self._reference_observations[
+            'walker/reference_rel_root_pos_local']))
+    # pylint: enable=g-long-lambda
     self._walker.observables.add_observable(
         'reference_appendages_pos',
         base_observable.Generic(self.get_reference_appendages_pos))
+
+    if enabled_reference_observables:
+      for name, observable in self.observables.items():
+        observable.enabled = name in enabled_reference_observables
+    self._walker.observables.add_observable(
+        'clip_id', base_observable.Generic(self.get_clip_id))
     self._walker.observables.add_observable(
         'velocimeter_control', base_observable.Generic(self.get_veloc_control))
     self._walker.observables.add_observable(
@@ -253,6 +258,14 @@ class ReferencePosesTask(composer.Task):
         'joints_vel_control',
         base_observable.Generic(self.get_joints_vel_control))
 
+    observables = []
+    observables += self._walker.observables.proprioception
+    observables += self._walker.observables.kinematic_sensors
+    observables += self._walker.observables.dynamic_sensors
+
+    for observable in observables:
+      observable.enabled = True
+
   def _get_possible_starts(self):
     # List all possible (clip, step) starting points.
     self._possible_starts = []
@@ -325,7 +338,7 @@ class ReferencePosesTask(composer.Task):
     # Set the walker at the beginning of the clip.
     self._set_walker(physics)
     self._walker_features = utils.get_features(physics, self._walker)
-    self._walker_features_prev = utils.get_features(physics, self._walker)
+    self._walker_features_prev = self._walker_features.copy()
 
     self._walker_joints = np.array(physics.bind(self._walker.mocap_joints).qpos)  # pytype: disable=attribute-error
 
@@ -338,6 +351,8 @@ class ReferencePosesTask(composer.Task):
                         'This is likely due to a proto/walker mismatch.'))
 
     self._update_ghost(physics)
+    self._reference_observations.update(
+        self.get_all_reference_observations(physics))
 
     # reset reward channels
     self._reset_reward_channels()
@@ -372,6 +387,13 @@ class ReferencePosesTask(composer.Task):
 
     self._walker_features_prev = self._walker_features.copy()
 
+  def after_compile(self, physics: 'mjcf.Physics',
+                    random_state: np.random.RandomState):
+    # populate reference observations field to initialize observations.
+    if not self._reference_observations:
+      self._reference_observations.update(
+          self.get_all_reference_observations(physics))
+
   def should_terminate_episode(self, physics: 'mjcf.Physics'):
     del physics  # physics unused by should_terminate_episode.
 
@@ -395,10 +417,10 @@ class ReferencePosesTask(composer.Task):
   def get_reference_rel_joints(self, physics: 'mjcf.Physics'):
     """Observation of the reference joints relative to walker."""
     del physics  # physics unused by reference observations.
-
     time_steps = self._time_step + self._ref_steps
-    return (self._clip_reference_features['joints'][time_steps] -
-            self._walker_joints).flatten()
+    diff = (self._clip_reference_features['joints'][time_steps] -
+            self._walker_joints)
+    return diff[:, self._walker.mocap_to_observable_joint_order].flatten()
 
   def get_reference_rel_bodies_pos_global(self, physics: 'mjcf.Physics'):
     """Observation of the reference bodies relative to walker."""
@@ -431,18 +453,21 @@ class ReferencePosesTask(composer.Task):
                   self._walker_features['body_positions'])[:, self._body_idxs])
     return np.concatenate([o.flatten() for o in obs])
 
-  def get_reference_ego_bodies_quats(self, physics: 'mjcf.Physics'):
+  def get_reference_ego_bodies_quats(self, unused_physics: 'mjcf.Physics'):
     """Body quat of the reference relative to the reference root quat."""
-    del physics  # physics unused by reference observations.
-
     time_steps = self._time_step + self._ref_steps
     obs = []
+    quats_for_clip = self._reference_ego_bodies_quats[self._current_clip_index]
     for t in time_steps:
-      for b in self._body_idxs:
-        obs.append(
-            tr.quat_diff(
-                self._clip_reference_features['quaternion'][t, :],
-                self._clip_reference_features['body_quaternions'][t, b, :]))
+      if t not in quats_for_clip:
+        root_quat = self._clip_reference_features['quaternion'][t, :]
+        quats_for_clip[t] = [
+            tr.quat_diff(  # pylint: disable=g-complex-comprehension
+                root_quat,
+                self._clip_reference_features['body_quaternions'][t, b, :])
+            for b in self._body_idxs
+        ]
+      obs.extend(quats_for_clip[t])
     return np.concatenate([o.flatten() for o in obs])
 
   def get_reference_rel_root_quat(self, physics: 'mjcf.Physics'):
@@ -498,7 +523,8 @@ class ReferencePosesTask(composer.Task):
 
     joints_curr, joints_prev = (self._walker_features['joints'],
                                 self._walker_features_prev['joints'])
-    return (joints_curr - joints_prev) / self._control_timestep
+    return (joints_curr - joints_prev)[
+        self._walker.mocap_to_observable_joint_order]/self._control_timestep
 
   def get_clip_id(self, physics: 'mjcf.Physics'):
     """Observation of the clip id."""
@@ -531,13 +557,16 @@ class ReferencePosesTask(composer.Task):
     return reference_observations
 
   def get_reward(self, physics: 'mjcf.Physics') -> float:
-    reference_observations = self.get_all_reference_observations(physics)
     reward, unused_debug_outputs, reward_channels = self._reward_fn(
-        self._termination_error,
-        self._termination_error_threshold,
+        termination_error=self._termination_error,
+        termination_error_threshold=self._termination_error_threshold,
         reference_features=self._current_reference_features,
         walker_features=self._walker_features,
-        reference_observations=reference_observations)
+        reference_observations=self._reference_observations)
+
+    if 'actuator_force' in self._reward_keys:
+      reward_channels['actuator_force'] = -self._actuator_force_coeff*np.mean(
+          np.square(self._walker.actuator_force(physics)))
 
     self._should_truncate = self._termination_error > self._termination_error_threshold
 
@@ -595,19 +624,21 @@ class MultiClipMocapTracking(ReferencePosesTask):
 
   def __init__(
       self,
-      walker: Union['base.Walker', Callable],  # pylint: disable=g-bare-generic
+      walker: Callable[..., 'legacy_base.Walker'],
       arena: composer.Arena,
       ref_path: Text,
       ref_steps: Sequence[int],
       dataset: Union[Text, Sequence[Any]],
       termination_error_threshold: float = 0.3,
       min_steps: int = 10,
-      reward_type: Text = 'default',
+      reward_type: Text = 'termination_reward',
       physics_timestep: float = DEFAULT_PHYSICS_TIMESTEP,
       always_init_at_clip_start: bool = False,
       proto_modifier: Optional[Any] = None,
       ghost_offset: Optional[Sequence[Union[int, float]]] = None,
       body_error_multiplier: Union[int, float] = 1.0,
+      actuator_force_coeff: float = 0.015,
+      enabled_reference_observables: Optional[Sequence[Text]] = None,
   ):
     """Mocap tracking task.
 
@@ -635,8 +666,11 @@ class MultiClipMocapTracking(ReferencePosesTask):
         the reference pose at the specified position offset.
       body_error_multiplier: A multiplier that is applied to the body error term
         when determining failure termination condition.
+      actuator_force_coeff: A coefficient for the actuator force reward channel.
+      enabled_reference_observables: Optional iterable of enabled observables.
+        If not specified, a reasonable default set will be enabled.
     """
-    super(MultiClipMocapTracking, self).__init__(
+    super().__init__(
         walker=walker,
         arena=arena,
         ref_path=ref_path,
@@ -649,14 +683,16 @@ class MultiClipMocapTracking(ReferencePosesTask):
         always_init_at_clip_start=always_init_at_clip_start,
         proto_modifier=proto_modifier,
         ghost_offset=ghost_offset,
-        body_error_multiplier=body_error_multiplier)
+        body_error_multiplier=body_error_multiplier,
+        actuator_force_coeff=actuator_force_coeff,
+        enabled_reference_observables=enabled_reference_observables)
     self._walker.observables.add_observable(
         'time_in_clip',
         base_observable.Generic(self.get_normalized_time_in_clip))
 
   def after_step(self, physics: 'mjcf.Physics', random_state):
     """Update the data after step."""
-    super(MultiClipMocapTracking, self).after_step(physics, random_state)
+    super().after_step(physics, random_state)
     self._time_step += 1
 
     # Update the walker's data for this timestep.
@@ -675,6 +711,9 @@ class MultiClipMocapTracking(ReferencePosesTask):
     # Terminate based on the error.
     self._end_mocap = self._time_step == self._last_step
 
+    self._reference_observations.update(
+        self.get_all_reference_observations(physics))
+
     self._update_ghost(physics)
 
   def get_normalized_time_in_clip(self, physics: 'mjcf.Physics'):
